JSCell

Struct JSCell 

pub struct JSCell<T> {
    inner: UnsafeCell<T>,
}
A cell for interior mutability, that (ab)uses NoGC as a borrow token for ensuring safety.

If inner type is Copy one should use std::cell::Cell instead.

Examples

use mozjs::context::NoGC;
use mozjs::cell::JSCell;
fn f(no_gc: &mut NoGC, cell: &JSCell<String>) {
    {
        let borrow = cell.borrow(no_gc);
    } // borrow goes out of scope here
    cell.set(no_gc, "Hello".to_string());
    {
       let borrow_mut = cell.borrow_mut(no_gc);
    } // borrow_mut goes out of scope here
}

Non-Examples

These fail to compile:

use mozjs::context::NoGC;
use mozjs::cell::JSCell;
fn mut_borrow_after_borrow(no_gc: &mut NoGC, cell: &JSCell<String>) {
    let borrow = cell.borrow(no_gc);
    cell.set(no_gc, "Hello".to_string());
    drop(borrow); // otherwise compiler can shorten borrow's lifetime
}
use mozjs::context::NoGC;
use mozjs::cell::JSCell;
fn mut_borrow_after_mut_borrow(no_gc: &mut NoGC, cell: &JSCell<String>) {
    let borrow_mut = cell.borrow_mut(no_gc);
    cell.set(no_gc, "Hello".to_string());
    drop(borrow_mut); // otherwise compiler can shorten borrow_mut's lifetime
}
use mozjs::context::NoGC;
use mozjs::cell::JSCell;
fn borrow_after_mut_borrow(no_gc: &mut NoGC, cell: &JSCell<String>) {
    let borrow_mut = cell.borrow_mut(no_gc);
    let borrow = cell.borrow(no_gc);
    drop(borrow_mut); // otherwise compiler can shorten borrow_mut's lifetime
}

Fields

inner: UnsafeCell<T>

Implementations

Source

impl<T> JSCell<T>

pub fn new(val: T) -> Self

pub fn set<'a, 'cx>(&'a self, _exclusive: &'cx mut NoGC, val: T)

Sets the wrapped value.

By passing a &mut NoGC we statically ensure no other borrows are alive at the same time.

For more information see JSCell documentation.

pub fn borrow_mut<'a: 'r, 'cx: 'r, 'r>( &'a self, _exclusive: &'cx mut NoGC, ) -> &'r mut T

Mutably borrows the wrapped value.

By passing a &mut NoGC we statically ensure no other borrows are alive at the same time.

For more information see JSCell documentation.

pub fn borrow<'a: 'r, 'no_cx: 'r, 'r>(&'a self, _no_gc: &'no_cx NoGC) -> &'r T

Immutably borrows the wrapped value.

By passing a &NoGC we statically ensure no mutable borrows are alive at the same time.

For more information see JSCell documentation.